@@ -11,6 +11,7 @@
 #include <fcntl.h>
 #include <string.h>
 #include <ctype.h>
+#include <freetype/freetype.h>
 #include "gf-pfb.h"
 
 static guchar * gf_pfb_search_def_name (const guchar * b, gint length, const guchar * name);
@@ -22,6 +23,8 @@ static void gf_pfb_search_def_bbox (const guchar * b, gint length, const guchar
 static const guchar * gf_pfb_search_def (const guchar * b, gint length, const gchar * name);
 static const guchar * gf_pfb_next_token (const guchar * b, const guchar * end);
 
+extern FT_Library ft_library;
+
 GFPFB *
 gf_pfb_open (const gchar * name)
 {
@@ -32,12 +35,15 @@ gf_pfb_open (const gchar * name)
 	guchar *buf;
 	guchar *f;
 	gint length;
+	gboolean ascii;
+	FT_Face ft_face;
+	FT_Error ft_result;
 
 	g_return_val_if_fail (name != NULL, NULL);
 
 	if (stat (name, &s) != 0) return NULL;
 	/* I think no font can be < 4096 bytes */
-	if (s.st_size < 4096) return NULL;
+	if (s.st_size < 8192) return NULL;
 	/* We cannot use more than 64K anyways */
 	size = MIN (s.st_size, 65536 + 6);
 
@@ -51,23 +57,33 @@ gf_pfb_open (const gchar * name)
 	if (!buf) return NULL;
 	if (buf == (gpointer) -1) return NULL;
 
-	if ((buf[0] != 0x80) && (buf[1] != 0x01)) {
+	if ((buf[0] == 0x80) && (buf[1] == 0x01)) {
+		/* Seems to be pfb font */
+		/* Well, we probably mess with some bytes */
+		if (strncmp (buf + 6, "%!PS-AdobeFont-1.", 17)) {
+			munmap (buf, size);
+			return NULL;
+		}
+		f = buf + 6;
+		length = 0x100 * buf[3] + buf[2];
+		ascii = FALSE;
+	} else {
+		/* We do not support pfa fonts, because certain version of FT2 may crash on some */
 		munmap (buf, size);
 		return NULL;
 	}
 
-	/* Well, we probably mess with some bytes */
-
-	if (strncmp (buf + 6, "%!PS-AdobeFont-1.", 17) != 0) {
-		munmap (buf, size);
+	ft_result = FT_New_Face (ft_library, name, 0, &ft_face);
+	if (ft_result != FT_Err_Ok) return NULL;
+	if (!FT_IS_SCALABLE (ft_face) ||
+	    !ft_face->family_name) {
+		FT_Done_Face (ft_face);
 		return NULL;
 	}
-
-	f = buf + 6;
-	length = 0x100 * buf[3] + buf[2];
+	FT_Done_Face (ft_face);
 
 	pfb = g_new0 (GFPFB, 1);
-
+	pfb->ascii = ascii;
 	pfb->filename = g_strdup (name);
 
 	/* Fill GlobalFontInfo */
@@ -92,7 +108,9 @@ gf_pfb_open (const gchar * name)
 
 	munmap (buf, size);
 
-	if ((!pfb->gfi.fontName) || (!pfb->gfi.fullName) || (!pfb->gfi.familyName)) {
+	if ((!pfb->gfi.fontName) ||
+	    (!pfb->gfi.fullName) ||
+	    (!pfb->gfi.familyName)) {
 		gf_pfb_close (pfb);
 		return NULL;
 	}
