@@ -11,31 +11,37 @@
  *
  */
 
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
 #include <errno.h>
 #include <string.h>
 #include <stdio.h>
+#include <locale.h>
 #include <popt-gnome.h>
 #include <glib.h>
 /* I know, that is is not nice, but that is exactly, what xml-config gives us */
 #include <parser.h>
 #include <xmlmemory.h>
 /* End of ugly thing */
+#include <freetype/freetype.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-i18n.h>
 #include "parseAFM.h"
 #include "gf-fontmap.h"
 #include "gf-pfb.h"
+#include "gf-ttf.h"
 
 /* Known file types */
 
 typedef enum {
 	GFI_FILE_UNKNOWN,
 	GFI_FILE_PFB,
-	GFI_FILE_AFM
+	GFI_FILE_PFA,
+	GFI_FILE_AFM,
+	GFI_FILE_TTF
 } GFIFileType;
 
 /* Known data about any font file */
@@ -46,6 +52,7 @@ typedef struct {
 	gchar *name;
 	gchar *familyname;
 	gchar *version;
+	gint face;
 } GFIFileData;
 
 typedef struct {
@@ -59,13 +66,17 @@ typedef struct {
 	gchar *psname;
 	GSList *afm_list;
 	GSList *pfb_list;
+	GSList *ttf_list;
 } GFIFontData;
 
+static void gfi_verify_fontmap (GFFontMap *map);
 static void gfi_verify_font_entry (GFFontEntry *e);
 static void gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f);
 static void gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f);
-static GFIFileData * gfi_read_afm_file_data (const guchar *name);
-static GFIFileData * gfi_read_pfb_file_data (const guchar *name);
+static void gfi_verify_ttf_file (GFFontEntry *e, GFFileEntry *f);
+static GFIFileData *gfi_read_afm_file_data (const guchar *name);
+static GFIFileData *gfi_read_pfb_file_data (const guchar *name);
+static void gfi_read_and_register_ttf_file_data (const guchar *name);
 static gboolean gfi_test_file_changed (GFFileEntry * f);
 
 static void gfi_read_aliases (const guchar *path);
@@ -75,12 +86,14 @@ static void gfi_scan_path (const guchar *path, gint level);
 static void gfi_try_font_file (const guchar *path);
 
 static void gfi_sort_fonts (void);
+GSList *gfi_sort_file_list (GSList *list, GFIFileType type);
 
 static void gfi_process_aliases (void);
 static void gfi_process_alias (GFIAliasData *a);
 
 static void gfi_build_fonts (void);
 static void gfi_build_font (GFIFontData * fd);
+static void gfi_build_ttf_font (GFIFontData *fd);
 
 static gboolean gfi_font_is_registered (const guchar *name);
 
@@ -105,6 +118,7 @@ static guchar * gfi_get_species_name (const guchar *fullname, const guchar *fami
  */
 
 static gboolean gfi_debug = FALSE;
+static gboolean gfi_smart = FALSE;
 static gboolean gfi_recursive = FALSE;
 static gboolean gfi_clean = FALSE;
 static gboolean gfi_usermap = TRUE;
@@ -117,6 +131,8 @@ static void add_path (poptContext ctx, enum poptCallbackReason reason, const str
 static const struct poptOption options[] = {
 	{ "debug", 'd', POPT_ARG_NONE, &gfi_debug, 0,
 	  N_("Print out debugging information"), NULL },
+	{ "smart", 'q', POPT_ARG_NONE, &gfi_smart, 0,
+	  N_("Tries to do everything automatically"), NULL },
 	{ "recursive", 'r', POPT_ARG_NONE, &gfi_recursive, 0,
 	  N_("Search directories recursively for font files"), NULL },
 	{ "clean", 'c', POPT_ARG_NONE, &gfi_clean, 0,
@@ -136,21 +152,50 @@ static const struct poptOption options[] = {
 	{ NULL, '\0', 0, NULL, 0 }
 };
 
-#if 0
-static GSList * fontmappath_list = NULL;
-static GSList * afmpath_list = NULL;
-static GSList * pfbpath_list = NULL;
-static GHashTable * assignment_dict = NULL;
-#endif
+/* Well-known font paths */
+static const guchar *smart_paths[] = {
+        /* generic */
+        "/usr/share/fonts",
+        "/usr/local/share/fonts",
+	DATADIR "/fonts",
+        /* Ghostscript */
+        "/usr/share/ghostscript/fonts",
+        "/usr/lib/ghostscript/fonts",
+        "/usr/local/share/ghostscript/fonts",
+        "/usr/local/lib/ghostscript/fonts",
+        "/usr/freeware/share/ghostscript/fonts",
+	DATADIR "/ghostscript/fonts",
+	LIBDIR "/ghostscript/fonts",
+        /* Debian */
+        "/usr/lib/texmf/fonts/type1/adobe",
+        "/usr/lib/texmf/fonts/type1/omega",
+        "/usr/lib/texmf/fonts/afm",
+        "/usr/share/texmf/fonts/type1/adobe",
+        "/usr/share/texmf/fonts/type1/omega",
+        "/usr/share/texmf/fonts/afm",
+        "/usr/share/a2ps",
+        /* Groff */
+        "/usr/share/groff/font",
+        "/usr/lib/groff/font",
+	DATADIR "/groff/font",
+	LIBDIR "/groff/font",
+        /* X */
+        "/usr/X11R6/lib/X11/fonts"
+	/* Misc */
+};
+#define NUM_SMART_PATHS (sizeof (smart_paths) / sizeof (smart_paths[0]))
+
 static GSList *aliaspath_list = NULL;
 
 /* List of GFIAliasData structures */
 static GSList *alias_list = NULL;
 
-static GSList * goodafm_list = NULL;
-static GHashTable * goodafm_dict = NULL;
-static GSList * goodpfb_list = NULL;
-static GHashTable * goodpfb_dict = NULL;
+static GSList *goodafm_list = NULL;
+static GHashTable *goodafm_dict = NULL;
+static GSList *goodpfb_list = NULL;
+static GHashTable *goodpfb_dict = NULL;
+static GSList *goodttf_list = NULL;
+static GHashTable *goodttf_dict = NULL;
 
 static GSList * font_list = NULL;
 static GHashTable * font_dict = NULL;
@@ -162,9 +207,12 @@ static GFFontDB *masterdb;
 /* Existing master map and target fontmap */
 static GFFontMap *mastermap, *newmap;
 
+FT_Library ft_library;
+
 int main (int argc, char ** argv)
 {
 	poptContext ctx;
+	FT_Error ft_result;
 	GFFontDB *db;
 	char ** args;
 	FILE *of;
@@ -175,6 +223,7 @@ int main (int argc, char ** argv)
 
 	goodafm_dict = g_hash_table_new (g_str_hash, g_str_equal);
 	goodpfb_dict = g_hash_table_new (g_str_hash, g_str_equal);
+	goodttf_dict = g_hash_table_new (g_str_hash, g_str_equal);
 	font_dict = g_hash_table_new (g_str_hash, g_str_equal);
 	goodfont_dict = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -185,13 +234,45 @@ int main (int argc, char ** argv)
 	result = poptGetNextOpt (ctx);
 	if (result != -1) { 
                 fprintf(stderr, "%s: %s: %s\n",
-                        "gnome-print-install",
+                        "gnome-font-install",
                         poptBadOption(ctx, POPT_BADOPTION_NOALIAS),
                         poptStrerror(result));
                 return 1;
 	}
 	args = (char **) poptGetArgs (ctx);
 
+	/* Initialize FreeType library */
+	ft_result = FT_Init_FreeType (&ft_library);
+	if (ft_result != FT_Err_Ok) {
+		fprintf (stderr, "Cannot initialize FreeType library\n");
+		return 1;
+	}
+
+	setlocale (LC_NUMERIC, "C");
+
+	/* Step 0: Smart */
+	if (gfi_smart) {
+		uid_t euid;
+		/* Set appropriate mode */
+		gfi_staticmap = gfi_dynamicmap = gfi_usermap = FALSE;
+		euid = geteuid ();
+		if (!euid) {
+			/* root - use dynamic map */
+			gfi_dynamicmap = TRUE;
+		} else {
+			gfi_usermap = TRUE;
+		}
+		gfi_staticmap = FALSE;
+		/* Set recursive */
+		gfi_recursive = TRUE;
+		/* Not clean */
+		gfi_clean = FALSE;
+		/* Set target */
+		gfi_target = NULL;
+		/* Set aliases */
+		aliaspath_list = g_slist_prepend (aliaspath_list, g_strdup (DATADIR "/fonts/adobe-urw.font"));
+	}
+
 	/* Step 1: Read existing fontmap */
 	if (gfi_debug) fprintf (stderr, "Reading fontmap... ");
 	if (gfi_clean) {
@@ -218,15 +299,13 @@ int main (int argc, char ** argv)
 	}
 	g_assert (mastermap != NULL);
 
-	/* Verify files from selected map, remove fonts */
-	if (gfi_debug) fprintf (stderr, "Verifying fontmap entries ");
-	while (mastermap->fonts) {
-		GFFontEntry *e;
-		e = mastermap->fonts;
-		mastermap->fonts = e->next;
-		gfi_verify_font_entry (e);
-		if (gfi_debug) fprintf (stderr, ".");
-	}
+	/* Verify fonts */
+	if (gfi_debug) fprintf (stderr, "Verifying fontmap entries\n");
+	gfi_verify_fontmap (db->usermaps);
+	gfi_verify_fontmap (db->dynamicmaps);
+	gfi_verify_fontmap (db->staticmaps);
+	/* Vacuum mastermap */
+	mastermap->fonts = NULL;
 	if (gfi_debug) fprintf (stderr, "Done\n");
 
 	/* Process alias files */
@@ -239,11 +318,18 @@ int main (int argc, char ** argv)
 	if (gfi_debug) fprintf (stderr, "Done\n");
 
 	/* Process directories/fontmaps */
-	if (gfi_debug) fprintf (stderr, "Scanning directories: ");
+	if (gfi_debug) fprintf (stderr, "Scanning paths\n");
 	for (i = 0; args && args[i]; i++) {
-		if (gfi_debug) fprintf (stderr, "%s ", args[i]);
+		if (gfi_debug) fprintf (stderr, "Scanning path %s\n", args[i]);
 		gfi_scan_path (args[i], 0);
 	}
+	if (gfi_smart) {
+		if (gfi_debug) fprintf (stderr, "Scanning smart paths\n");
+		for (i = 0; i < NUM_SMART_PATHS; i++) {
+			if (gfi_debug) fprintf (stderr, "Scanning path %s\n", smart_paths[i]);
+			gfi_scan_path (smart_paths[i], 0);
+		}
+	}
 	if (gfi_debug) fprintf (stderr, "Done\n");
 
 	/* Free popt context */
@@ -302,6 +388,21 @@ int main (int argc, char ** argv)
 	return 0;
 }
 
+static void
+gfi_verify_fontmap (GFFontMap *map)
+{
+	while (map) {
+		GFFontEntry *e;
+		if (gfi_debug) fprintf (stderr, "Map: %s ", map->path);
+		for (e = map->fonts; e != NULL; e = e->next) {
+			gfi_verify_font_entry (e);
+			if (gfi_debug) fprintf (stderr, ".");
+		}
+		if (gfi_debug) fprintf (stderr, "\n");
+		map = map->next;
+	}
+}
+
 /*
  * Process GFFontEntry
  *
@@ -316,9 +417,9 @@ gfi_verify_font_entry (GFFontEntry *e)
 {
 	switch (e->type) {
 	case GF_FONT_ENTRY_TYPE1:
-		gfi_verify_afm_file (e, &e->files[0]);
-		gfi_verify_pfb_file (e, &e->files[1]);
-		if (strcmp (e->files[0].psname, e->files[0].psname)) {
+		if (e->files[1].path) gfi_verify_afm_file (e, &e->files[1]);
+		gfi_verify_pfb_file (e, &e->files[0]);
+		if (e->files[1].psname && strcmp (e->files[1].psname, e->files[0].psname)) {
 			GFIAliasData *a;
 			/* We are aliased entry */
 			a = g_new (GFIAliasData, 1);
@@ -328,7 +429,7 @@ gfi_verify_font_entry (GFFontEntry *e)
 		}
 		break;
 	case GF_FONT_ENTRY_TRUETYPE:
-		/* No TrueType support at moment */
+		gfi_verify_ttf_file (e, &e->files[0]);
 		break;
 	default:
 		g_assert_not_reached ();
@@ -338,7 +439,7 @@ gfi_verify_font_entry (GFFontEntry *e)
 /*
  * Verifies afm entry
  *
- * Checkes, whether file is intact
+ * Checks, whether file is intact
  * (Indirectly) If not intact, or no data, try to parse it
  * (Directly, Indirectly) If good, create new GFIFileData
  * and save it to goodafm_list/goodafm_dict
@@ -413,6 +514,46 @@ gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f)
 	}
 }
 
+/* Same as previous for ttf files */
+
+static void
+gfi_verify_ttf_file (GFFontEntry *e, GFFileEntry *f)
+{
+	GFIFileData *fd;
+	gchar *key;
+
+	/* Test, whether we are already verified and registered */
+	key = g_strdup_printf ("%s:%d", f->path, e->face);
+	fd = g_hash_table_lookup (goodttf_dict, key);
+	if (fd) return;
+
+	if (!gfi_test_file_changed (f)) {
+		fd = g_new0 (GFIFileData, 1);
+		fd->type = GFI_FILE_TTF;
+		fd->entry.path = g_strdup (f->path);
+		fd->entry.size = f->size;
+		fd->entry.mtime = f->mtime;
+		fd->entry.psname = g_strdup (f->psname);
+		fd->name = g_strdup (e->name);
+		fd->familyname = g_strdup (e->familyname);
+		fd->version = g_strdup (e->version);
+		fd->face = e->face;
+	} else {
+		gfi_read_and_register_ttf_file_data (f->path);
+		return;
+	}
+
+	if (fd) {
+		goodttf_list = g_slist_prepend (goodttf_list, fd);
+		g_hash_table_insert (goodttf_dict, key, fd);
+	} else {
+		g_free (key);
+#ifdef GFI_VERBOSE
+		if (gfi_debug) fprintf (stderr, "TTF Not good: %s\n", f->name);
+#endif
+	}
+}
+
 /*
  * Tries to parse afm file, if sucessful return GFIFileData
  * Return NULL if file does not exist/cannot be parsed
@@ -495,6 +636,68 @@ gfi_read_pfb_file_data (const guchar * name)
 	return fd;
 }
 
+/*
+ * NOTICE - ttf parsing is more complex due to collections
+ */
+
+static void
+gfi_read_and_register_ttf_file_data (const guchar *name)
+{
+	GFIFileData *fd;
+	struct stat s;
+	GFTTF *ttf;
+	const guchar *familyname, *stylename;
+
+	if (stat (name, &s) < 0) return;
+	if (!S_ISREG (s.st_mode)) return;
+	if (!g_path_is_absolute (name)) return;
+
+	/* Hack to recognize the only free Japanese TTF font */
+	if (!strcmp (name + strlen (name) - 19, "/wadalab-gothic.ttf")) {
+		familyname = "Wadalab Gothic";
+		stylename = NULL;
+	} else {
+		familyname = NULL;
+		stylename = NULL;
+	}
+
+	ttf = gf_ttf_open (name, 0, familyname, stylename);
+	if (ttf) {
+		gint num_faces, face;
+		num_faces = ttf->num_faces;
+		face = 0;
+		while (ttf) {
+			guchar *key;
+
+			fd = g_new (GFIFileData, 1);
+
+			fd->type = GFI_FILE_TTF;
+			fd->entry.path = g_strdup (name);
+			fd->entry.size = s.st_size;
+			fd->entry.mtime = s.st_mtime;
+			fd->entry.psname = g_strdup (ttf->gfi.fontName);
+			fd->name = g_strdup (ttf->gfi.fullName);
+			fd->familyname = g_strdup (ttf->gfi.familyName);
+			fd->version = g_strdup (ttf->gfi.version);
+			fd->face = face;
+
+			goodttf_list = g_slist_prepend (goodttf_list, fd);
+			key = g_strdup_printf ("%s:%d", name, face);
+			g_hash_table_insert (goodttf_dict, key, fd);
+
+			gf_ttf_close (ttf);
+
+			face += 1;
+
+			if (face < num_faces) {
+				ttf = gf_ttf_open (name, face, familyname, stylename);
+			} else {
+				ttf = NULL;
+			}
+		}
+	}
+}
+
 /*
  * Checks, whether file has been changed, according to its
  * size and mtime fields
@@ -516,7 +719,9 @@ gfi_test_file_changed (GFFileEntry *f)
 }
 
 /*
- * Scan directory for font files, update goodafm and goodpfb
+ * Scan path
+ * Path can be either directory, in which case it is read (recursively, if needed), or
+ * file, in which case it is just tested.
  */
 
 #define MAX_RECURSION_DEPTH 32
@@ -527,7 +732,7 @@ gfi_scan_path (const guchar *path, gint level)
 	static ino_t inodes[MAX_RECURSION_DEPTH + 1];
 	struct stat s;
 
-	if (level > MAX_RECURSION_DEPTH) return;
+	if (level >= MAX_RECURSION_DEPTH) return;
 
 	if (gfi_debug) fprintf (stderr, "Scanning path %s (%s)\n", path, gfi_recursive ? "recursive" : "non-recursive");
 
@@ -600,7 +805,7 @@ gfi_scan_path (const guchar *path, gint level)
 static void
 gfi_try_font_file (const guchar *fn)
 {
-	GFIFileData * fd;
+	GFIFileData *fd;
 	struct stat s;
 	gchar *name;
 
@@ -615,13 +820,9 @@ gfi_try_font_file (const guchar *fn)
 		name = g_strdup (fn);
 	}
 
-	fd = g_hash_table_lookup (goodafm_dict, name);
-	if (fd) {
-		g_free (name);
-		return;
-	}
-	fd = g_hash_table_lookup (goodpfb_dict, name);
-	if (fd) {
+	if (g_hash_table_lookup (goodafm_dict, name) ||
+	    g_hash_table_lookup (goodpfb_dict, name) ||
+	    g_hash_table_lookup (goodttf_dict, name)) {
 		g_free (name);
 		return;
 	}
@@ -644,6 +845,8 @@ gfi_try_font_file (const guchar *fn)
 		return;
 	}
 
+	gfi_read_and_register_ttf_file_data (name);
+
 	g_free (name);
 	/* Cannot read :( */
 }
@@ -659,40 +862,46 @@ gfi_try_font_file (const guchar *fn)
 static void
 gfi_sort_fonts (void)
 {
-	GFIFileData *file;
-	GFIFontData *font;
+	goodafm_list = gfi_sort_file_list (goodafm_list, GFI_FILE_AFM);
+	goodpfb_list = gfi_sort_file_list (goodpfb_list, GFI_FILE_PFB);
+	goodttf_list = gfi_sort_file_list (goodttf_list, GFI_FILE_TTF);
+}
 
-	while (goodafm_list) {
-		file = (GFIFileData *) goodafm_list->data;
+GSList *
+gfi_sort_file_list (GSList *list, GFIFileType type)
+{
+	while (list) {
+		GFIFileData *file;
+		GFIFontData *font;
+		file = (GFIFileData *) list->data;
 		font = g_hash_table_lookup (font_dict, file->name);
 		if (!font) {
 			font = g_new (GFIFontData, 1);
 			font->name = g_strdup (file->name);
 			font->familyname = g_strdup (file->familyname);
 			font->psname = g_strdup (file->entry.psname);
-			font->afm_list = font->pfb_list = NULL;
+			font->afm_list = font->pfb_list = font->ttf_list = NULL;
 			font_list = g_slist_prepend (font_list, font);
 			g_hash_table_insert (font_dict, font->name, font);
 		}
-		font->afm_list = g_slist_prepend (font->afm_list, file);
-		goodafm_list = g_slist_remove (goodafm_list, file);
+		switch (type) {
+		case GFI_FILE_AFM:
+			font->afm_list = g_slist_prepend (font->afm_list, file);
+			break;
+		case GFI_FILE_PFB:
+			font->pfb_list = g_slist_prepend (font->pfb_list, file);
+			break;
+		case GFI_FILE_TTF:
+			font->ttf_list = g_slist_prepend (font->ttf_list, file);
+			break;
+		default:
+			g_warning ("Unknown file type encountered in list: %d", type);
+			break;
+		}
+		list = g_slist_remove (list, file);
 	}
 
-	while (goodpfb_list) {
-		file = (GFIFileData *) goodpfb_list->data;
-		font = g_hash_table_lookup (font_dict, file->name);
-		if (!font) {
-			font = g_new (GFIFontData, 1);
-			font->name = g_strdup (file->name);
-			font->familyname = g_strdup (file->familyname);
-			font->psname = g_strdup (file->entry.psname);
-			font->afm_list = font->pfb_list = NULL;
-			font_list = g_slist_prepend (font_list, font);
-			g_hash_table_insert (font_dict, font->name, font);
-		}
-		font->pfb_list = g_slist_prepend (font->pfb_list, file);
-		goodpfb_list = g_slist_remove (goodpfb_list, file);
-	}
+	return list;
 }
 
 /*
@@ -792,7 +1001,7 @@ gfi_process_alias (GFIAliasData *a)
 		return;
 	}
 
-	new = g_new (GFFontEntry, 1);
+	new = g_new0 (GFFontEntry, 1);
 	new->next = NULL;
 	new->type = GF_FONT_ENTRY_TYPE1;
 	new->name = g_strdup (afmfd->name);
@@ -801,9 +1010,9 @@ gfi_process_alias (GFIAliasData *a)
 	new->speciesname = gfi_get_species_name (new->name, new->familyname);
 	new->weight = g_strdup (fi->gfi->weight);
 	/* AFM */
-	new->files[0] = afm->entry;
+	new->files[1] = afm->entry;
 	/* PFB */
-	new->files[1] = pfb->entry;
+	new->files[0] = pfb->entry;
 	/* Misc */
 	new->italicangle = fi->gfi->italicAngle;
 
@@ -833,15 +1042,19 @@ gfi_build_font (GFIFontData *fd)
 	GFFontEntry *new;
 	GFIFileData *afmdata, *pfbdata;
 	gdouble afmversion, pfbversion;
-	GSList * l;
-	FILE * f;
-	int status;
-	Font_Info * fi;
+	GSList *l;
+	GFPFB *pfb;
 
 	/* Return if we are already registered */
 	/* Fixme: We should free structs */
 	if (g_hash_table_lookup (goodfont_dict, fd->name)) return;
 
+	/* Are we already registered in some master fontmap? */
+	if (gfi_font_is_registered (fd->name)) {
+		if (gfi_debug) fprintf (stderr, "Font %s is already registered\n", fd->name);
+		return;
+	}
+
 	pfbdata = NULL;
 	pfbversion = -1e18;
 
@@ -857,8 +1070,11 @@ gfi_build_font (GFIFontData *fd)
 		}
 	}
 
-	/* If we do not have pfb file return */
-	if (!pfbdata) return;
+	/* If we do not have pfb file try to build ttf entry */
+	if (!pfbdata) {
+		gfi_build_ttf_font (fd);
+		return;
+	}
 
 	afmdata = NULL;
 	afmversion = -1e18;
@@ -876,51 +1092,98 @@ gfi_build_font (GFIFontData *fd)
 		}
 	}
 
-	/* If we do not have afm file return */
-	if (!afmdata) return;
+	/* Seems, that we have everything */
+	pfb = gf_pfb_open (pfbdata->entry.path);
+	if (!pfb) return;
 
-	/* We have to read afm to get weight and italicangle */
+	new = g_new0 (GFFontEntry, 1);
+	new->next = NULL;
+	new->type = GF_FONT_ENTRY_TYPE1;
+	new->name = g_strdup (fd->name);
+	new->version = g_strdup (pfbdata->version);
+	new->familyname = g_strdup (pfb->gfi.familyName);
+	new->speciesname = gfi_get_species_name (new->name, new->familyname);
+	new->weight = g_strdup (pfb->gfi.weight);
+	/* PFB */
+	new->files[0] = pfbdata->entry;
+	/* AFM */
+	if (afmdata) new->files[1] = afmdata->entry;
+	/* Misc */
+	new->italicangle = pfb->gfi.italicAngle;
 
-	fi = NULL;
-	f = fopen (afmdata->entry.path, "r");
-	/* This shouldn't happen */
-	if (!f) return;
-	status = parseFile (f, &fi, P_G);
-	fclose (f);
-	/* This shouldn't happen! */
-	if (status != AFM_ok) {
-		if (fi) parseFileFree (fi);
-		return;
-	}
+	/* Release PFB info */
+	gf_pfb_close (pfb);
 
-	/* Now we should have everything we need */
+	/* Register it */
+	if (gfi_debug) fprintf (stderr, "Registered font: %s\n", new->name);
 
-	/* Final check - test, whether given font is already registered */
+	goodfont_list = g_slist_prepend (goodfont_list, new);
+	g_hash_table_insert (goodfont_dict, new->name, new);
+}
+
+static void
+gfi_build_ttf_font (GFIFontData *fd)
+{
+	GFFontEntry *new;
+	GFIFileData *ttfdata;
+	gdouble ttfversion;
+	GSList * l;
+	GFTTF *ttf;
+
+	/* Return if we are already registered */
+	/* Fixme: We should free structs */
+	if (g_hash_table_lookup (goodfont_dict, fd->name)) return;
+
+	/* Are we already registered in some master fontmap? */
 	if (gfi_font_is_registered (fd->name)) {
 		if (gfi_debug) fprintf (stderr, "Font %s is already registered\n", fd->name);
 		return;
 	}
 
-	new = g_new (GFFontEntry, 1);
-	new->next = NULL;
-	new->type = GF_FONT_ENTRY_TYPE1;
-	new->name = g_strdup (fd->name);
-	new->version = g_strdup (afmdata->version);
-	new->familyname = g_strdup (fi->gfi->familyName);
-	new->speciesname = gfi_get_species_name (new->name, new->familyname);
-	new->weight = g_strdup (fi->gfi->weight);
-	/* AFM */
-	new->files[0] = afmdata->entry;
-	/* PFB */
-	new->files[1] = pfbdata->entry;
-	/* Misc */
-	new->italicangle = fi->gfi->italicAngle;
+	ttfdata = NULL;
+	ttfversion = -1e18;
 
-	/* Release AFM info */
-	parseFileFree (fi);
+	/* Find ttf vith highest version */
+	for (l = fd->ttf_list; l != NULL; l = l->next) {
+		GFIFileData *d;
+		gdouble v;
+		d = (GFIFileData *) l->data;
+		v = (d->version) ? atof (d->version) : 1.0;
+		if (v > ttfversion) {
+			ttfversion = v;
+			ttfdata = d;
+		}
+	}
+
+	/* If we do not have ttf data, return */
+	if (!ttfdata) return;
+
+	/* Read ttf again to get Weight and italic */
+
+	ttf = gf_ttf_open (ttfdata->entry.path, ttfdata->face, fd->familyname, NULL);
+	if (!ttf) return;
+
+	/* Now we should have everything we need */
+
+	new = g_new0 (GFFontEntry, 1);
+	new->next = NULL;
+	new->type = GF_FONT_ENTRY_TRUETYPE;
+	new->name = g_strdup (fd->name);
+	new->version = g_strdup (ttfdata->version);
+	new->familyname = g_strdup (fd->familyname);
+	new->speciesname = gfi_get_species_name (new->name, new->familyname);
+	new->weight = g_strdup (ttf->gfi.weight);
+	/* TTF */
+	new->files[0] = ttfdata->entry;
+	new->face = ttfdata->face;
+	/* Misc */
+	new->italicangle = ttf->gfi.italicAngle;
+
+	/* Release TTF info */
+	gf_ttf_close (ttf);
 
 	/* Register it */
-	if (gfi_debug) fprintf (stderr, "Registered font: %s\n", new->name);
+	if (gfi_debug) fprintf (stderr, "Registered TTF font: %s\n", new->name);
 
 	goodfont_list = g_slist_prepend (goodfont_list, new);
 	g_hash_table_insert (goodfont_dict, new->name, new);
@@ -1061,33 +1324,56 @@ gfi_write_font (xmlNodePtr root, GFFontEntry *e)
 	n = xmlNewDocNode (root->doc, NULL, "font", NULL);
 	xmlAddChild (root, n);
 
-	/* Set format */
-	if (strcmp (e->files[0].psname, e->files[1].psname)) {
-		xmlSetProp (n, "format", "type1alias");
+	if (e->type == GF_FONT_ENTRY_TYPE1) {
+		/* Set format */
+		if (e->files[0].psname && e->files[1].psname && strcmp (e->files[0].psname, e->files[1].psname)) {
+			xmlSetProp (n, "format", "type1alias");
+			xmlSetProp (n, "alias", e->files[1].psname);
+		} else {
+			xmlSetProp (n, "format", "type1");
+		}
+
+		if (e->files[1].path) {
+			/* afm file */
+			f = xmlNewDocNode (root->doc, NULL, "file", NULL);
+			xmlAddChild (n, f);
+			xmlSetProp (f, "type", "afm");
+			xmlSetProp (f, "path", e->files[1].path);
+			g_snprintf (c, 128, "%d", (gint) e->files[1].size);
+			xmlSetProp (f, "size", c);
+			g_snprintf (c, 128, "%d", (gint) e->files[1].mtime);
+			xmlSetProp (f, "mtime", c);
+		}
+
+		/* pfb file */
+		f = xmlNewDocNode (root->doc, NULL, "file", NULL);
+		xmlAddChild (n, f);
+		xmlSetProp (f, "type", "pfb");
+		xmlSetProp (f, "path", e->files[0].path);
+		g_snprintf (c, 128, "%d", (gint) e->files[0].size);
+		xmlSetProp (f, "size", c);
+		g_snprintf (c, 128, "%d", (gint) e->files[0].mtime);
+		xmlSetProp (f, "mtime", c);
+	} else if (e->type == GF_FONT_ENTRY_TRUETYPE) {
+		/* Set format */
+		xmlSetProp (n, "format", "truetype");
+		g_snprintf (c, 128, "%d", e->face);
+		xmlSetProp (n, "subface", c);
+
+		/* ttf file */
+		f = xmlNewDocNode (root->doc, NULL, "file", NULL);
+		xmlAddChild (n, f);
+		xmlSetProp (f, "type", "ttf");
+		xmlSetProp (f, "path", e->files[0].path);
+		g_snprintf (c, 128, "%d", (gint) e->files[0].size);
+		xmlSetProp (f, "size", c);
+		g_snprintf (c, 128, "%d", (gint) e->files[0].mtime);
+		xmlSetProp (f, "mtime", c);
 	} else {
-		xmlSetProp (n, "format", "type1");
+		/* Set format */
+		xmlSetProp (n, "format", "unknown");
 	}
 
-	/* afm file */
-	f = xmlNewDocNode (root->doc, NULL, "file", NULL);
-	xmlAddChild (n, f);
-	xmlSetProp (f, "type", "afm");
-	xmlSetProp (f, "path", e->files[0].path);
-	g_snprintf (c, 128, "%d", (gint) e->files[0].size);
-	xmlSetProp (f, "size", c);
-	g_snprintf (c, 128, "%d", (gint) e->files[0].mtime);
-	xmlSetProp (f, "mtime", c);
-
-	/* pfb file */
-	f = xmlNewDocNode (root->doc, NULL, "file", NULL);
-	xmlAddChild (n, f);
-	xmlSetProp (f, "type", "pfb");
-	xmlSetProp (f, "path", e->files[1].path);
-	g_snprintf (c, 128, "%d", (gint) e->files[1].size);
-	xmlSetProp (f, "size", c);
-	g_snprintf (c, 128, "%d", (gint) e->files[1].mtime);
-	xmlSetProp (f, "mtime", c);
-
 	/* Other properties */
 	xmlSetProp (n, "name", e->name);
 	xmlSetProp (n, "version", e->version);
@@ -1097,10 +1383,6 @@ gfi_write_font (xmlNodePtr root, GFFontEntry *e)
 	xmlSetProp (n, "weight", e->weight);
 	g_snprintf (c, 128, "%g", e->italicangle);
 	xmlSetProp (n, "italicangle", c);
-
-	if (strcmp (e->files[0].psname, e->files[1].psname)) {
-		xmlSetProp (n, "alias", e->files[1].psname);
-	}
 }
 
 static guchar *
