@@ -38,8 +38,10 @@ static void gp_fontmap_load_dir (GPFontMap *map, const guchar *dirname);
 static void gp_fontmap_load_file (GPFontMap *map, const guchar *filename);
 static void gp_fm_load_font_2_0_type1 (GPFontMap *map, xmlNodePtr node);
 static void gp_fm_load_font_2_0_type1alias (GPFontMap *map, xmlNodePtr node);
+static void gp_fm_load_font_2_0_truetype (GPFontMap *map, xmlNodePtr node);
 static void gp_font_entry_2_0_load_data (GPFontEntry *e, xmlNodePtr node);
 static void gp_font_entry_2_0_type1_load_files (GPFontEntryT1 *t1, xmlNodePtr node);
+static void gp_font_entry_2_0_truetype_load_files (GPFontEntryTT *tt, xmlNodePtr node);
 static void gp_fm_load_fonts_2_0 (GPFontMap * map, xmlNodePtr root);
 
 static void gp_fontmap_ref (GPFontMap * map);
@@ -69,7 +71,6 @@ gp_fontmap_get (void)
 		/* If > 1 sec is passed from last query, check timestamps */
 		if ((time (NULL) > lastaccess) && gp_fm_is_changed (map)) {
 			/* Any directory is changed, so force rereading of map */
-			g_print ("Fontmap is changed, rereading\n");
 			gp_fontmap_release (map);
 			map = NULL;
 		}
@@ -107,7 +108,6 @@ gp_fontmap_unref (GPFontMap * map)
 	g_return_if_fail (map != NULL);
 
 	if (--map->refcount < 1) {
-		g_print ("Releasing fontmap\n");
 		if (map->familydict) g_hash_table_destroy (map->familydict);
 		if (map->fontdict) g_hash_table_destroy (map->fontdict);
 		if (map->familylist) {
@@ -467,6 +467,9 @@ gp_fm_load_fonts_2_0 (GPFontMap * map, xmlNodePtr root)
 				} else if (!strcmp (format, "type1alias")) {
 					/* We are type1/type1alias entry */
 					gp_fm_load_font_2_0_type1alias (map, child);
+				} else if (!strcmp (format, "truetype")) {
+					/* We are truetype entry */
+					gp_fm_load_font_2_0_truetype (map, child);
 				}
 				xmlFree (format);
 			}
@@ -500,12 +503,12 @@ gp_fm_load_font_2_0_type1 (GPFontMap *map, xmlNodePtr node)
 
 	gp_font_entry_2_0_load_data (e, node);
 	gp_font_entry_2_0_type1_load_files (t1, node);
-	if (!e->familyname || !e->psname || !t1->afm.name || !t1->pfb.name) {
+	if (!e->familyname || !e->psname || !t1->pfb.name) {
 		gp_font_entry_unref (e);
 		return;
 	}
 
-	t1->Weight = gp_fontmap_lookup_weight (e->weight);
+	e->Weight = gp_fontmap_lookup_weight (e->weight);
 
 	if (!e->speciesname) {
 		e->speciesname = gp_fm_get_species_name (e->name, e->familyname);
@@ -516,9 +519,9 @@ gp_fm_load_font_2_0_type1 (GPFontMap *map, xmlNodePtr node)
 		gchar *p;
 		p = strstr (e->speciesname, "Italic");
 		if (!p) p = strstr (e->speciesname, "Oblique");
-		t1->ItalicAngle = p ? -10.0 : 0.0;
+		e->ItalicAngle = p ? -10.0 : 0.0;
 	} else {
-		t1->ItalicAngle = atof (t);
+		e->ItalicAngle = atof (t);
 		xmlFree (t);
 	}
 
@@ -563,12 +566,12 @@ gp_fm_load_font_2_0_type1alias (GPFontMap *map, xmlNodePtr node)
 
 	gp_font_entry_2_0_load_data (e, node);
 	gp_font_entry_2_0_type1_load_files (t1, node);
-	if (!e->familyname || !e->psname || !t1->afm.name || !t1->pfb.name) {
+	if (!e->familyname || !e->psname || !t1->pfb.name) {
 		gp_font_entry_unref (e);
 		return;
 	}
 
-	t1->Weight = gp_fontmap_lookup_weight (e->weight);
+	e->Weight = gp_fontmap_lookup_weight (e->weight);
 
 	if (!e->speciesname) {
 		e->speciesname = gp_fm_get_species_name (e->name, e->familyname);
@@ -579,9 +582,9 @@ gp_fm_load_font_2_0_type1alias (GPFontMap *map, xmlNodePtr node)
 		gchar *p;
 		p = strstr (e->speciesname, "Italic");
 		if (!p) p = strstr (e->speciesname, "Oblique");
-		t1->ItalicAngle = p ? -10.0 : 0.0;
+		e->ItalicAngle = p ? -10.0 : 0.0;
 	} else {
-		t1->ItalicAngle = atof (t);
+		e->ItalicAngle = atof (t);
 		xmlFree (t);
 	}
 
@@ -590,6 +593,63 @@ gp_fm_load_font_2_0_type1alias (GPFontMap *map, xmlNodePtr node)
 	map->fonts = g_slist_prepend (map->fonts, e);
 }
 
+static void
+gp_fm_load_font_2_0_truetype (GPFontMap *map, xmlNodePtr node)
+{
+	GPFontEntryTT *tt;
+	GPFontEntry *e;
+	xmlChar *xmlname, *t;
+
+	/* Get our unique name */
+	xmlname = xmlGetProp (node, "name");
+	/* Check, whether we are already registered */
+	if (g_hash_table_lookup (map->fontdict, xmlname)) {
+		xmlFree (xmlname);
+		return;
+	}
+
+	tt = g_new0 (GPFontEntryTT, 1);
+	e = (GPFontEntry *) tt;
+
+	e->type = GP_FONT_ENTRY_TRUETYPE;
+	e->refcount = 1;
+	e->face = NULL;
+	e->name = g_strdup (xmlname);
+	xmlFree (xmlname);
+
+	gp_font_entry_2_0_load_data (e, node);
+	gp_font_entry_2_0_truetype_load_files (tt, node);
+	if (!e->familyname || !e->psname || !tt->ttf.name) {
+		gp_font_entry_unref (e);
+		return;
+	}
+
+	e->Weight = gp_fontmap_lookup_weight (e->weight);
+
+	if (!e->speciesname) {
+		e->speciesname = gp_fm_get_species_name (e->name, e->familyname);
+	}
+
+	t = xmlGetProp (node, "italicangle");
+	if (t == NULL) {
+		gchar *p;
+		p = strstr (e->speciesname, "Italic");
+		if (!p) p = strstr (e->speciesname, "Oblique");
+		e->ItalicAngle = p ? -10.0 : 0.0;
+	} else {
+		e->ItalicAngle = atof (t);
+		xmlFree (t);
+	}
+
+	t = xmlGetProp (node, "subface");
+	tt->facenum = (t) ? atoi (t) : 0;
+	if (t) xmlFree (t);
+
+	g_hash_table_insert (map->fontdict, e->name, e);
+	map->num_fonts++;
+	map->fonts = g_slist_prepend (map->fonts, e);
+}
+
 /* Loads common font property data */
 
 static void
@@ -619,17 +679,39 @@ gp_font_entry_2_0_type1_load_files (GPFontEntryT1 *t1, xmlNodePtr node)
 			xmlChar *type;
 			/* We are <file> node */
 			type = xmlGetProp (child, "type");
-			if (!strcmp (type, "afm") && !t1->afm.name) {
+			if (type && !strcmp (type, "afm") && !t1->afm.name) {
 				t1->afm.name = gp_xmlGetPropString (child, "path");
-			} else if (!strcmp (type, "pfb") && !t1->pfb.name) {
+			} else if (type && !strcmp (type, "pfb") && !t1->pfb.name) {
 				t1->pfb.name = gp_xmlGetPropString (child, "path");
 			}
-			xmlFree (type);
+			if (type) xmlFree (type);
 		}
 		if (t1->afm.name && t1->pfb.name) return;
 	}
 }
 
+/* Loads "afm" and "pfb" file nodes */
+
+static void
+gp_font_entry_2_0_truetype_load_files (GPFontEntryTT *tt, xmlNodePtr node)
+{
+	xmlNodePtr child;
+
+	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
+		/* Scan all children nodes */
+		if (!strcmp (child->name, "file")) {
+			xmlChar *type;
+			/* We are <file> node */
+			type = xmlGetProp (child, "type");
+			if (type && !strcmp (type, "ttf") && !tt->ttf.name) {
+				tt->ttf.name = gp_xmlGetPropString (child, "path");
+			}
+			if (type) xmlFree (type);
+		}
+		if (tt->ttf.name) return;
+	}
+}
+
 #if 0
 static void
 gp_fm_load_font_2_0 (GPFontMap * map, xmlNodePtr node, gboolean alias)
